<?php
/**
 * ClipBox MediaWiki extension  version 1.0.0
 *	for details please see: https://www.mediawiki.org/wiki/Extension:ClipBox
 *
 * Copyright (c) 2024 Kimagurenote https://kimagurenote.net/
 * License: Revised BSD license http://opensource.org/licenses/BSD-3-Clause
 *
 * Function:
 *	Mediawiki extension that embed input box for copy to clipboard.
 *
 * Usage:
 *	<clipbox input="readonly">CampaignCode</clipbox>
 *	for details please see: https://kimagurenote.net/kn/ClipBox
 *
 * History:
 * 2024.02.01 Version 1.0.0
 *	1st version
 *
 * @file
 * @ingroup Extensions
 * @author Kimagurenote
 * @copyright © 2024 Kimagurenote
 * @license The BSD 3-Clause License
 */

if( !defined( 'MEDIAWIKI' ) ) {
	echo( "This file is an extension to the MediaWiki software and cannot be used standalone.\n" );
	die( 1 );
}

class ClipBox {
	/**
	 * Register all the new tags with the Parser.
	 *
	 * @param Parser &$parser
	 */
	public static function onParserFirstCallInit( Parser $parser ) {
		/* default tag */
		$parser->setHook( 'clipbox', [ self::class, 'embedClipBox' ] );

		return true;
	}


	/**
	 * @link https://www.mediawiki.org/wiki/Manual:Hooks/LinkerMakeExternalLink
	 * @param string $input
	 * @param array $argv
	 * @param Parser $parser
	 *
	 * @return string
	 */
	public static function embedClipBox( $input, $argv, $parser ) {

		$divattrs['class'] = "clipbox";
		if( !empty( $argv['divclass'] ) ) {
			$divattrs['class'] .= ' ' . $divattrs['divclass'];
		}
		if( !empty( $argv['title'] ) ) {
			$divattrs['title'] = $argv['title'];
		}
		if( !empty( $argv['style'] ) ) {
			$divattrs['style'] = $argv['style'];
		}

		if( empty( $argv['id'] ) ) {
			$inputattrs['id'] = "ClipBoxCopyTarget";
		} else {
			$inputattrs['id'] = $argv['id'];
		}
		$inputattrs['type'] = "text";
		$inputattrs['value'] = htmlspecialchars( $input );
		if( !empty( $argv['size'] ) ) {
			$inputattrs['size'] = $argv['size'];
		}
		if( !empty( $argv['input'] ) ) {
			foreach( explode( ' ', $argv['input'] ) as $ia ) {
				if( $ia and empty( $inputattrs[$ia] ) ) {
					$inputattrs[$ia] = "";
				}
			}
		}

		$precapattrs['id'] = $inputattrs['id'] . "PreCaption";
		$precapattrs['type'] = "hidden";
		$precapattrs['value'] = $argv['title'] ? wfMessage( 'clipbox-precaption', $argv['title'] )->inContentLanguage()->plain() : "";

		$postcapattrs['id'] = $inputattrs['id'] . "PostCaption";
		$postcapattrs['type'] = "hidden";
		$postcapattrs['value'] = wfMessage( 'clipbox-postcaption' )->inContentLanguage()->plain();

		$buttonattrs['onclick'] = "javascript:ClipBoxCopyToClipboard('" . $inputattrs['id'] . "')";
		$inputattrs['ondblclick'] = $buttonattrs['onclick'];
		$buttoncaption = wfMessage( 'clipbox-button' )->inContentLanguage()->plain();

		$ret = Html::rawElement( 'div', $divattrs,
			Html::rawElement( 'input', $inputattrs ) .
			Html::rawElement( 'input', $precapattrs ) .
			Html::rawElement( 'input', $postcapattrs ) .
			Html::rawElement( 'button', $buttonattrs, $buttoncaption )
		);

$ret .= <<<'START_END_MARKER'

<script type="text/javascript">
function ClipBoxCopyToClipboard( btn ) {
	box = document.getElementById(btn);
	precap = document.getElementById(btn+"PreCaption");
	postcap = document.getElementById(btn+"PostCaption");
	if( box ) {
		if( window.clipboardData ) {
			window.clipboardData.setData("Text", box.value);
		} else {
			navigator.clipboard.writeText(box.value);
		}
		if( postcap ) {
			message = box.value + postcap.value;
			if( precap ) {
				message = precap.value + message;
			}
			alert( message );
		}
	}
}
</script>
START_END_MARKER;

		return $ret;
	}
}
